@@ -9,9 +9,7 @@
 # Distributed under the terms of the GNU Public Licence (GPL).
 
 
-# add one argument like 1 to make it verbose
-
-# do locking, not loosing files because someone redownloaded the index files
+# do locking, not losing files because someone redownloaded the index files
 # right then
 # use IO::Handle;
 
@@ -28,27 +26,37 @@ my $nice_mode=0;
 my $verbose=0;
 my $help;
 my $force;
+my $sim_mode=0;
+my $offline=0;
 
 my %options = (
     "h|help" => \$help,
-    "n|nice-mode=s"     => \$nice_mode,
-    "v|verbose"           => \$verbose,
-    "f|force"           => \$force,
-    "c|config-file=s"        => \$configfile
+    "n|nice" => \$nice_mode,
+    "v|verbose" => \$verbose,
+    "f|force" => \$force,
+    "c|config-file=s" => \$configfile,
+    "s|simulate" => \$sim_mode,
+    "o|offline" => \$offline
 );
 
 
 &help unless ( GetOptions(%options));
+if ($sim_mode) {
+  $verbose = 1;
+  print "Simulation mode. Just printing what would be done.\n";
+}
 &help if ($help);
 
 $configfile=abs_path($configfile);
 
 sub help {
     die <<EOM
-    Usage: $0 [ -n ] [ -v ] [ -c configfile ]
+    Usage: $0 [ -n ] [ -s|v ] [ -o ] [ -f ] [ -c configfile ]
     -n : nice mode, refresh index files first, then renice to 20 and continue
+    -s : simulate mode, just print what would be done to package files
+    -o : offline mode, don't update index files. Overrides offline_mode from configfile
     -v : verbose mode
-    -f : force executing, disable santity checks
+    -f : force executing, disable sanity checks
 EOM
     ;
 }
@@ -61,41 +69,48 @@ sub printmsg {
 #############################################################################
 ### configuration ###########################################################
 # Include the library for the config file parser
-require '/usr/share/apt-cacher/apt-cacher-lib.pl';
+push @INC,'/usr/share/apt-cacher/';
+require 'apt-cacher-lib.pl';
 # Read in the config file and set the necessary variables
 
-my $configref;
+# $cfg needs to be global for setup_ownership
+our $cfg;
+
 eval {
-        $configref = read_config($configfile);
+	 $cfg = read_config($configfile);
 };
-my %config = %$configref;
 
 # not sure what to do if we can't read the config file...
 die "Could not read config file: $@" if $@;
 
-my $globlockfile="$config{cache_dir}/private/exlock";
+my $globlockfile="$$cfg{cache_dir}/private/exlock";
+define_global_lockfile($globlockfile);
 
 # check whether we're actually meant to clean the cache
-if ( $config{clean_cache} ne 1 ) {
+if ( $$cfg{clean_cache} ne 1 ) {
 	exit 0;
 }
+# check offline mode in config
+if (defined $$cfg{offline_mode} && $$cfg{offline_mode}) {
+	$offline = 1;
+}
+# change uid and gid
+setup_ownership($cfg);
 
 #############################################################################
 
-my $refresh=1;
-
 my %valid;
 
-my $tempdir="$config{cache_dir}/temp";
+my $tempdir="$$cfg{cache_dir}/temp";
 mkdir $tempdir if !-d $tempdir;
 die "Could not create tempdir $tempdir\n" if !-d $tempdir;
 unlink (<$tempdir/*>);
 
 ### Preparation of the package lists ########################################
 
-chdir "$config{cache_dir}/packages" && -w "." || die "Could not enter the cache dir";
+chdir "$$cfg{cache_dir}/packages" && -w "." || die "Could not enter the cache dir";
 
-if($> == 0 && !$config{user} && !$force) {
+if($> == 0 && !$$cfg{user} && !$force) {
     die "Running $0 as root\nand no effective user has been specified. Aborting.\nPlease set the effective user in $configfile\n";
 }
 
@@ -110,18 +125,16 @@ sub get {
     print $fh "GET $path_info\r\nConnection: Close\r\n\r\n";
     close($fh);
     if($? && ! $force) {
-        die "Unable to update $path_info . Network problems?\nRun $0 with -v to get more details.\nCleanup aborted since cached data may be damaged.\n";
+	die "Unable to update $path_info . Network problems?\nRun $0 with -v to get more details.\nCleanup aborted since cached data may be damaged.\n";
     }
 }
 
 
 
-# file state decissions, lock that area
-open(my $lck, $globlockfile);
-flock($lck, LOCK_EX);
+# file state decisions, lock that area
+set_global_lock(": file state decision");
 my @ifiles=(<*es.gz>, <*es.bz2>, <*es>, <*_Index>);
-flock($lck, LOCK_UN);
-close($lck);
+release_global_lock();
 
 for (@ifiles) {
 
@@ -129,51 +142,62 @@ for (@ifiles) {
    $valid{$_}=1;
 
    # now refresh them, unless disabled by the setting above
-   if($refresh) {
-       printmsg "D: $_\n";
+   if(!$offline) {
+       printmsg "Downloading: $_\n";
       # if the path is stored there, better use that
       if(-s "../private/$_.complete") {
-         open(my $tmp, "../private/$_.complete");
-         my $url=<$tmp>;
-         &get($url);
-         close $tmp;
+	 open(my $tmp, "../private/$_.complete");
+	 my $url=<$tmp>;
+	 &get($url);
+	 close $tmp;
       }
       else {
-         my $tmp=$_;
-         $tmp=~s/^/\//;
-         $tmp=~s/_/\//g;
-         &get($tmp);
+	 my $tmp=$_;
+	 $tmp=~s/^/\//;
+	 $tmp=~s/_/\//g;
+	 &get($tmp);
       }
    }
+   else {
+     printmsg "Offline: Using existing $_\n";
+   }
 }
 
 setpriority 0, 0, 20 if $nice_mode;
 
 # use the list of config files we already know
-for my $file (@ifiles) { 
-    printmsg "R: $file\n";
+for my $file (@ifiles) {
+    printmsg "Reading: $file\n";
 
     # get both locks and create a temp. copy
     my $tmpfile= "$tempdir/$file";
-    open(my $glck, $globlockfile);
-    flock($glck, LOCK_EX);
-    open(my $lck, $file);
+    set_global_lock(": temporary copy");
+    open(my $lck, $file) || do {
+	release_global_lock();
+	print ("Error: cannot open $file for locking");
+	next;
+    };
     flock($lck, LOCK_EX);
-    link($file, $tmpfile);
+    link($file, $tmpfile) || do {
+	release_global_lock();
+	print ("Cannot link $file $tmpfile. Check permissions. $$cfg{cache_dir} must be single filesystem.\n");
+	next;
+    };
     flock($lck, LOCK_UN);
     close($lck);
-    flock($glck, LOCK_UN);
-    close($glck);
+    release_global_lock();
+
+    if(-e $tmpfile && -z $tmpfile && $tmpfile=~/(gz|bz2)$/) {
+	# moo, junk, empty file, most likely leftovers from previous versions
+	# of apt-cacher-cleanup where the junk was "protected" from being
+	# deleted. Purge later by not having in %valid.
+	# delete $valid{$file}; <- will be recreated RSN either way
+      die("Found empty index file $file. Delete this manually or use --force if the repository is no longer interesting. \nExiting to prevent deletion of cache contents.\n") unless $force;
+      print "Forced ignoring empty index file $file, apparently undownloadable. All packages referenced by it will be lost!\n";
 
-    if(-e $tmpfile && ! -s $tmpfile && $tmpfile=~/bz2$/) {
-        # moo, junk, empty file, most likely leftovers from previous versions
-        # of apt-cacher-cleanup where the junk was "protected" from beeing
-        # deleted. Purge later by not having in %valid.
-        # delete $valid{$file}; <- will be recreated RSN either way
-        print "Ignoring empty index file $file, apparently undownloadable\n" if $verbose;
     }
     else {
-        extract_sums($tmpfile, \%valid) || die("Error processing $file in $config{cache_dir}/packages, cleanup stopped.\nRemove the file if the repository is no longer interesting and the packages pulled from it are to be removed.\n");
+	extract_sums($tmpfile, \%valid) || die("Error processing $file in $$cfg{cache_dir}/packages, cleanup stopped.\nRemove the file if the repository is no longer interesting and the packages pulled from it are to be removed.\n");
     }
 }
 
@@ -184,29 +208,29 @@ for(<*.deb>, <*.udeb>, <*.bz2>, <*.gz>, <*.dsc>) {
     # should affect source packages but not index files which are added to the
     # valid list above
     if(! defined($valid{$_})) {
-        unlink $_, "../headers/$_", "../private/$_.complete";
-        printmsg "Removing source: $_ and company...\n";
+	unlink $_, "../headers/$_", "../private/$_.complete" unless $sim_mode;
+	printmsg "Removing source: $_ and company...\n";
     }
 }
 
 # similar thing for possibly remaining cruft
-chdir "$config{cache_dir}/headers" && -w "." || die "Could not enter the cache dir";
+chdir "$$cfg{cache_dir}/headers" && -w "." || die "Could not enter the cache dir";
 
-# headers for previosly expired files
+# headers for previously expired files
 for(<*.deb>, <*.bz2>, <*.gz>, <*.dsc>) {
    if(! defined($valid{$_})) {
-      unlink $_, "../private/$_.complete";
+      unlink $_, "../private/$_.complete" unless $sim_mode;
       printmsg "Removing expired headers: $_ and company...\n";
    }
 }
 
 # also remove void .complete files, created by broken versions of apt-cacher in rare conditions
-chdir "$config{cache_dir}/private" && -w "." || die "Could not enter the cache dir";
+chdir "$$cfg{cache_dir}/private" && -w "." || die "Could not enter the cache dir";
 for(<*.deb.complete>, <*.bz2.complete>, <*.gz.complete>, <*.dsc.complete>) {
    s/.complete$//;
    if(! (defined($valid{$_}) && -e "../packages/$_" && -e "../headers/$_") ) {
       printmsg "Removing: $_.complete\n";
-      unlink "$_.complete";
+      unlink "$_.complete" unless $sim_mode;
    }
 }
 
@@ -217,31 +241,29 @@ for(<*.notify>) {
     my @info = stat($_);
     # even the largest package should be downloadable in two days or so
     if(int(($now - $info[9])/3600) > 48) {
-        printmsg "Removing orphaned notify file: $_\n";
-        unlink $_;
+	printmsg "Removing orphaned notify file: $_\n";
+	unlink $_ unless $sim_mode;
     }
 }
 
-#define_global_lockfile("$config{cache_dir}/exlock");
 #&set_global_lock(": cleanup zombies");
 
-chdir "$config{cache_dir}/packages";
+chdir "$$cfg{cache_dir}/packages";
 
 for(<*>) {
-    # must be empty and not complete and beeing downloaded right now
-    if(!-s $_) {
-        my $fromfile;
-        if(open($fromfile, $_) && flock($fromfile, LOCK_EX|LOCK_NB)) {
-            # double-check, may have changed while locking
-            if(!-s $_) {
-                printmsg "Removing zombie files: $_ and company...\n";
-                unlink $_, "../headers/$_", "../private/$_.complete";
-                flock($fromfile, LOCK_UN);
-                close($fromfile);
-            }
-        }
+    # must be empty and not complete and being downloaded right now
+    if(-z $_) {
+	my $fromfile;
+	if(open($fromfile, $_) && flock($fromfile, LOCK_EX|LOCK_NB)) {
+	    # double-check, may have changed while locking
+	    if(-z $_) {
+		printmsg "Removing zombie files: $_ and company...\n";
+		unlink $_, "../headers/$_", "../private/$_.complete" unless $sim_mode;
+		flock($fromfile, LOCK_UN);
+		close($fromfile);
+	    }
+	}
     }
 }
 
 unlink (<$tempdir/*>);
-
