@@ -65,7 +65,7 @@ my %options = (
 
 my $cfg;
 eval {
-        $cfg = read_config($configfile);
+	$cfg = read_config($configfile);
 };
 
 # not sure what to do if we can't read the config file...
@@ -101,25 +101,25 @@ sub importrec {
     #print "Entering: $import_dir\n";
 
     if($recmode) {
-        my $cwd=Cwd::getcwd();
-        for(<*>) {
-            if(-d $_ && ! -l $_) {
-                importrec($_) if -d $_;
-                chdir $cwd;
-                #print "Back in $cwd\n";
-            }
-        }
+	my $cwd=Cwd::getcwd();
+	for(<*>) {
+	    if(-d $_ && ! -l $_) {
+		importrec($_) if -d $_;
+		chdir $cwd;
+		#print "Back in $cwd\n";
+	    }
+	}
     }
 
     ### Loop through all the .debs in the import dir
     foreach my $packagefile ( <*.deb>, <*.udeb>, <*.dsc>, <*.diff.gz>, <*_*tar.gz>, <*diff.bz2>, <*_*.tar.bz2> ) {
 
-        # Get some things we need to insert into the header
-        my $headerlength = (stat($packagefile))[7];
-        my $headeretag = int(rand(100000))."-".int(rand(1000))."-".int(rand(100000000));
-        $headeretag =~ s/^\s*(.*?)\s*$/$1/;
-        my $frompackagefile=$packagefile; # backup of the original name
-        $packagefile=~s/_\d+%3a/_/;
+	# Get some things we need to insert into the header
+	my $headerlength = (stat($packagefile))[7];
+	my $headeretag = int(rand(100000))."-".int(rand(1000))."-".int(rand(100000000));
+	$headeretag =~ s/^\s*(.*?)\s*$/$1/;
+	my $frompackagefile=$packagefile; # backup of the original name
+	$packagefile=~s/_\d+%3a/_/;
 
 	# Generate a header
 	my $httpheader = "HTTP/1.1 200 OK
@@ -136,32 +136,32 @@ Content-Type: application/x-debian-package
 "
 ; # there are TWO new lines
 
-        # Then cat the header to a temp file
-        print "Importing: $packagefile\n" if !$quiet;
-        unlink "$header_dir/$packagefile", "$target_dir/$packagefile",  "$private_dir/$packagefile.complete"; # just to be sure
-        if($symlink_mode) {
-            symlink(abs_path($frompackagefile), "$target_dir/$packagefile") ||
-            (unlink("$target_dir/$packagefile") && symlink(abs_path($frompackagefile), "$target_dir/$packagefile")) ||
-            die "Failed to create the symlink $target_dir/$packagefile";
-        }
-        elsif($ro_mode) {
-            link($frompackagefile, "$target_dir/$packagefile") || copy($frompackagefile, "$target_dir/$packagefile") || die "Failed to copy $frompackagefile";
-        }
-        else {
-            rename($frompackagefile, "$target_dir/$packagefile");
-        }
+	# Then cat the header to a temp file
+	print "Importing: $packagefile\n" if !$quiet;
+	unlink "$header_dir/$packagefile", "$target_dir/$packagefile",  "$private_dir/$packagefile.complete"; # just to be sure
+	if($symlink_mode) {
+	    symlink(abs_path($frompackagefile), "$target_dir/$packagefile") ||
+	    (unlink("$target_dir/$packagefile") && symlink(abs_path($frompackagefile), "$target_dir/$packagefile")) ||
+	    die "Failed to create the symlink $target_dir/$packagefile";
+	}
+	elsif($ro_mode) {
+	    link($frompackagefile, "$target_dir/$packagefile") || copy($frompackagefile, "$target_dir/$packagefile") || die "Failed to copy $frompackagefile";
+	}
+	else {
+	    rename($frompackagefile, "$target_dir/$packagefile") || die "Failed to rename $frompackagefile. Try read-only (-r) or symlink (-s) options.";
+	}
 
-        open(my $headfile, ">$header_dir/$packagefile");
-        print $headfile $httpheader;
-        close $headfile;
+	open(my $headfile, ">$header_dir/$packagefile");
+	print $headfile $httpheader;
+	close $headfile;
 
-        my $completefile = "$private_dir/$packagefile.complete";
-        open(MF, ">$completefile");
-        close(MF);
-        # copy the ownership of the private directory
-        chown $info[4], $info[5], "$header_dir/$packagefile", "$target_dir/$packagefile",  "$private_dir/$packagefile.complete";
+	my $completefile = "$private_dir/$packagefile.complete";
+	open(MF, ">$completefile");
+	close(MF);
+	# copy the ownership of the private directory
+	chown $info[4], $info[5], "$header_dir/$packagefile", "$target_dir/$packagefile",  "$private_dir/$packagefile.complete";
 
-        $packagesimported++;
+	$packagesimported++;
     }
 }
 
@@ -178,10 +178,9 @@ sub help {
 
 If -c is omited, '-c /etc/apt-cacher/apt-cacher.conf' is assumed.
 If package-source-dir is omited, the filename from apt-cacher.conf is used.
--R means descending into subdirectories while discovering pacakge files.
--r tells to not move the source files. Instead, hardlinks or real copies are
-   created.
--s tells to create symlinks to the source files and not move them. If the
+-R descend into subdirectories to find source package files.
+-r read but do not move the source files. Instead, create hardlinks or real copies.
+-s create symlinks to the source files and not move them. If the
    target symlink exists, it will be removed.
 ";
 }
